#!/usr/bin/Rscript --vanilla

## Profile Evaluator for SNORT AnomalyDetection v2.5

##quit status:
#  0 - OK
#  1 - help
#  2 - error  

##dependencies
#parsing script params
require('getopt', quietly=T);

#getopt specification
#  columns:
#     1  long flag name
#     2  short flag name
#     3  argument mask:  0 - no argument; 1 - requied argument; 2 - optional argument
#     4  argument data type: logical, integer, double, complex, character
#     5  (optional) brief description
spec <- matrix(c(
   'help',    'h',   0,  "logical",  'Show this help',
   'verbose',  'v',   0,  "logical",  'Verbose mode',
   'log',    'l',   1,  "character",'Log file path',
   'pattern',  'p',   1,  "character",'Pattern file (path)',
   'save',    's',   1,  "character",'Save MAE/M to file',
   'skip',    'S',   1,  "integer",  'Skip periods on the beginning of log file'
), ncol=5, byrow=TRUE);


## Shows help message
printHelp <- function(){
   write(c("Evaluator for SNORT AnomalyDetection\n",
  getopt(spec, command="ad_evaluator.r", usage=TRUE)
  ),stdout())
}

## Shows messages & errors
showMessage <- function(message, check.verbose=TRUE, error=FALSE){
  if(!is.character(message)) stop("message must be a 'character' type")

  message <- c("Evaluator:", message)
  
  if(error){
     message <- append(message, "ERROR:", 1)
     write(paste(message, collapse=" "), stderr())
  }else if(check.verbose){
    if(opt$verbose) write(paste(message, collapse=" "), stdout())
  } else {
    write(paste(message, collapse=" "), stdout())
  }
}

#parsing parameters
opt <- getopt(spec)

#if asked for help:
if ( !is.null(opt$help) ) {
  printHelp()
  q(status=1);
}

#set some defaults for the options that were not specified
if ( is.null(opt$verbose   ))  { opt$verbose  <- FALSE                }
if ( is.null(opt$log      ))  { opt$log     <- "czysty.csv"}
#if ( is.null(opt$pattern   ))  { opt$pattern  <- "<set here your default pattern file path>"}
if ( is.null(opt$skip     ))  { opt$skip    <- 0                   }

#jeśli nie podana ścieżka pattern
if(is.null(opt$pattern)){
   printHelp()
   showMessage("Nothing to do. Bye")
   quit(status=0)
}

if ( opt$skip < 0 ){
   showMessage("Value of '-s' option MUST be positive. Exiting", error=T)
   q(status=2)
}

showMessage("Starting Evaluator for SNORT AnomalyDetection")
showMessage(c("Using log file:", opt$log))
showMessage(c("Using pattern file:", opt$pattern))

##if log with header uncomment this:
log <- read.csv(opt$log)
real_cols<-grep("\\.", log)
log[,real_cols]<-log[,real_cols]*100
##else if log without header uncomment this:  
#log <- read.csv(opt$log,FALSE)
  
log.column.date <- 1
log.column.hour <- 2
log.column.day <- 3
log.column.interval <- 4

log.interval <- log[1,log.column.interval]

pattern <- read.csv(opt$pattern, FALSE)
real_cols<-grep("\\.", pattern)
pattern[,real_cols]<-pattern[,real_cols]*100

pattern.column.date <- 1
pattern.column.hour <- 3
pattern.column.day <- 2
pattern.column.interval <- 4
  
pattern.interval <- pattern[1, pattern.column.interval]
pattern.frequency <- 604800/pattern.interval

if(pattern.interval != log.interval){
  showMessage("Log and pattern intervals are different", error=T)
  quit(status=2)
}

if(ncol(pattern) != ncol(log)){
  showMessage("Log and pattern have different number of columns", error=T)
  quit(status=2)
} else if(ncol(pattern) == ((ncol(log)*2)-4)) {
  showMessage("Using profile file instead of pattern", error=T)
  
}

if (opt$skip > 0) { 
   showMessage(c("Skipping first ", opt$skip, "weeks in log")) 
   skip <- (pattern.frequency*opt$skip)+1
   log <- log[skip:nrow(log),]
}

##convert weekday to number(1...7)
log[,log.column.day]<-format(strptime(log[,log.column.date],"%d-%m-%y"),"%u")

#nazwy kolumn
save.names <- as.character(names(log))

pattern<-as.matrix(pattern)
log<-as.matrix(log)

if(pattern[1,pattern.column.date] == "0"){
   #profil tygodniowy
   #zostaje ostatni tydzień z loga
   log<-as.matrix(log[(nrow(log)-(pattern.frequency-1)):nrow(log),])
   
   showMessage(c("Start point:", log[1,log.column.date],log[1,log.column.hour]))
   showMessage(c("End point:", log[nrow(log),log.column.date],log[nrow(log),log.column.hour]))
   
   #dopasowanie początku plików:
   if ( (log[1,log.column.day]!=pattern[1,pattern.column.day])||(log[1,log.column.hour]!=pattern[1,pattern.column.hour]) ){
      log.starting.point <- intersect(which(log[,log.column.day] == pattern[1,pattern.column.day]),
                which(log[,log.column.hour] == pattern[1,pattern.column.hour]))
      
      log2 <- matrix(0,nrow=nrow(log), ncol=ncol(log))
      log2[1:(nrow(log)-log.starting.point+1),] <-log[log.starting.point:nrow(log),]  
      log2[(nrow(log)-log.starting.point+2):nrow(log),] <-log[1:(log.starting.point-1),]
      log<-log2
      #pliki zaczynają i kończą się w tym samym miejscu w czasie
   }
} else {
   #profil z datami - albo pattern albo predicted pattern   
   #daty wspólne dla obydwóch plików
   same.dates <- intersect(log[,log.column.date],pattern[,pattern.column.date])
   
   if ( length(same.dates)==0 ){
      showMessage("Log and pattern do not contain common elements", error=T)
     quit(status=2)
   }
   
   #znalezienie pierwszej godziny wspólnej dla wybranego dnia początkowego
   log.start.hours<-as.character(log[which(log[,log.column.date]==same.dates[1]),log.column.hour])
   pattern.start.hours<-as.character(pattern[which(pattern[,pattern.column.date]==same.dates[1]),pattern.column.hour])
   start.hour<-intersect(log.start.hours,pattern.start.hours)[1]
   
   if ( length(start.hour)==0 ){
      showMessage("Log and pattern do not contain common elements", error=T)
     quit(status=2)
   }
   
   showMessage(c("Start point:", same.dates[1],start.hour))
   
   #znalezienie ostatniej godziny wspólnej dla wybranego dnia końcowego
   log.end.hours<-as.character(log[which(log[,log.column.date]==same.dates[length(same.dates)]),log.column.hour])
   pattern.end.hours<-as.character(pattern[which(pattern[,pattern.column.date]==same.dates[length(same.dates)]),pattern.column.hour])
   end.hour<-intersect(log.end.hours,pattern.end.hours)
   end.hour<-end.hour[length(end.hour)]
   
   showMessage(c("End point:", same.dates[length(same.dates)],end.hour))
   
   #przycięcie loga
   log.start.point <- intersect(which(log[,log.column.date]==same.dates[1]),which(log[,log.column.hour]==start.hour))
   log.end.point <- intersect(which(log[,log.column.date]==same.dates[length(same.dates)]),which(log[,log.column.hour]==end.hour))
   log <- log[log.start.point:log.end.point,]
   
   #przycięcie pattern
   pattern.start.point <- intersect(which(pattern[,pattern.column.date]==same.dates[1]),which(pattern[,pattern.column.hour]==start.hour))
   pattern.end.point <- intersect(which(pattern[,pattern.column.date]==same.dates[length(same.dates)]),which(pattern[,pattern.column.hour]==end.hour))
   pattern <- pattern[pattern.start.point:pattern.end.point,] 
}

##matrix for errors array
errors <- matrix(0,nrow=nrow(log), ncol=ncol(log))

#liczenie modułów błędów
for (column.log in 5:ncol(log)){
   errors[,column.log] <- abs(as.numeric(log[,column.log])-as.numeric(pattern[,column.log]))
}

save.data <- matrix(0,ncol=ncol(errors), nrow=4)
save.data[1,]<-save.names
save.data[1,1]<-"Value"
save.data[2,1]<-"MAE"
save.data[3,1]<-"M"
save.data[4,1]<-"MAE/M"

for(x in 5:ncol(errors)){
     save.data[2,x] <- mean(errors[,x])
     save.data[3,x] <- mean(as.numeric(log[,x]))
}

save.data[4,5:ncol(save.data)] <- as.numeric(save.data[2,5:ncol(save.data)]) / as.numeric(save.data[3,5:ncol(save.data)])

save.data <- subset(save.data, select=-c(2:4))

showMessage("MAE/M value for each counter:",FALSE)
print(save.data)

##saving files
if (is.character(opt$save)) { 
write.table(save.data, file=opt$save, sep=",", col.names=FALSE, row.names=FALSE, append=FALSE, quote=FALSE)
}

showMessage(c("Total time elapsed:", as.character(proc.time()[3]),"sec"))
